==> dasu_2.02.jar: edu/northwestern/dasu/measurement/SignalManipulation.java <==
/**
 * Dasu Project
 *
 * File:         Mathematics.java
 * RCS:          $Id: SignalManipulation.java,v 1.2 2010/11/01 15:37:00 mas939 Exp $
 * Description:  Mathematics class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Oct 10, 2010 at 4:13:22 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.measurement
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.measurement;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;

import org.apache.commons.math.stat.univariate.rank.Percentile;

import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.measurement.NetstatBWStats.NetstatBWData;

/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The Mathematics class ...
 */
public class SignalManipulation {

    /*
     * Does not exist for signals with variance equal to zero
     */
    public static Float getPearsonCorrelation(Float[] scores1,Float[] scores2){
        Double result = 0d;
        Float sum_sq_x = 0f;
        Float sum_sq_y = 0f;
        Float sum_coproduct = 0f;
        Float mean_x = scores1[0];
        Float mean_y = scores2[0];
        for(int i=2;i<scores1.length+1;i+=1){
            Float sweep =Float.valueOf(i-1)/i;
            Float delta_x = scores1[i-1]-mean_x;
            Float delta_y = scores2[i-1]-mean_y;
            sum_sq_x += delta_x * delta_x * sweep;
            sum_sq_y += delta_y * delta_y * sweep;
            sum_coproduct += delta_x * delta_y * sweep;
            mean_x += delta_x / i;
            mean_y += delta_y / i;
        }
       
        Double pop_sd_x = (Double) Math.sqrt(sum_sq_x/scores1.length);
        Double pop_sd_y = (Double) Math.sqrt(sum_sq_y/scores1.length);
        Float cov_x_y = sum_coproduct / scores1.length;
        result = cov_x_y / (pop_sd_x*pop_sd_y);
        return result.floatValue();
    }
    
    public static Float getPearsonCorrelation(Collection<Float> s1, Collection<Float> s2){
    	
    	Float[] scores1 = new Float[s1.size()];
    	scores1 = s1.toArray(scores1);
    	Float[] scores2 = new Float[s2.size()];
    	scores2 = s2.toArray(scores2);

    	return getPearsonCorrelation(scores1, scores2);
    		
    }
    
	public static Collection<Float> getCorrelationCollection(Float[] signal1, Float[] signal2, int wz) {
		
		LinkedList<Float> detected = new LinkedList<Float>();
		
		if (((signal1.length != signal2.length) || (signal1.length) < wz) || (wz < 1)) {
			System.out.println("ERROR: signals are of different length or specified window size is invalid!");
			return null;
		}
		
		Float[] s1 = new Float[wz+1];
		Float[] s2 = new Float[wz+1];
		
    	for (int pos=0; pos < signal1.length - wz; pos++) {
    		//System.out.println("pos: " + pos);
    		
    		Object o1[] = getSubSignalWindow(signal1, pos, pos+wz);
    		for (int i=0; i< o1.length; i++) s1[i] = (Float) o1[i];
    		
    		Object o2[] = getSubSignalWindow(signal2, pos, pos+wz);
    		for (int i=0; i< o2.length; i++) s2[i] = (Float) o2[i];
    		
    		detected.add(getPearsonCorrelation(s1, s2));
    	}
    	
    	return detected;
    		
    }
    
	public static Collection<Float> getCorrelationCollection(Collection<Float> signal1, Collection<Float> signal2, int wz) {
		
		Float[] s1 = new Float[wz];
		Float[] s2 = new Float[wz];
		s1 = signal1.toArray(s1);
		s2 = signal2.toArray(s2);
		
      	return getCorrelationCollection(s1, s2, wz);
    		
    }
	
//	public static Collection<Float> getCumulative(Collection<Float> signal) {
//		
//		LinkedList<Float> ret = new LinkedList<Float>();
//		Iterator<Float> iter = signal.iterator();
//		Float prev = 0f;
//		while (iter.hasNext()) {
//			Float val = iter.next();
//			ret.add(prev+val);
//			prev += val;
//		}
//		
//		return ret;
//		
//	}
	
	public static Collection getCumulative(Collection signal) {
		
		LinkedList ret = new LinkedList();
		Iterator iter = signal.iterator();
		Object obj = iter.next();
		if ( obj instanceof Float) {

			Float prev = (Float) obj;
			ret.add(prev);
			
			while (iter.hasNext()) {
				Float val = (Float) iter.next();
				ret.add(prev+val);
				prev += val;
			}
		}
		
		if ( obj instanceof Long) {

			Long prev = (Long) obj;
			ret.add(prev);
			
			while (iter.hasNext()) {
				Long val = (Long) iter.next();
				ret.add(prev+val);
				prev += val;
			}
		}
		
		return ret;
		
	}
	
	public static Collection<Float> getCumulative(Float[] signal) {
		return getCumulative(new ArrayList(Arrays.asList(signal)));
	}

	public static Collection<Long> getCumulative(Long[] signal) {
		return getCumulative(new ArrayList(Arrays.asList(signal)));
	}
	
    public static Float getSlope(Float[] signal,Long[] time){
    	
    	if (time[time.length-1] == time[0])
    		return null;
    	
    	return (signal[signal.length-1] - signal[0]) / (time[time.length-1] - time[0]);
    }
	
	public static Collection<Float> getSlopeCollection(Float[] signal, Long[] time, int wz) {
		
		LinkedList<Float> detected = new LinkedList<Float>();
		
		if (((signal.length != time.length) || (signal.length) < wz) || (wz < 2)) {
			System.out.println("ERROR: signals are of different length or specified window size is invalid!");
			return null;
		}
		
		Float[] s1 = new Float[wz+1];
		Long[] t1 = new Long[wz+1];
		
    	for (int pos=0; pos < signal.length - wz; pos++) {
    		
    		Object o1[] = getSubSignalWindow(signal, pos, pos+wz);
    		for (int i=0; i< o1.length; i++) s1[i] = (Float) o1[i];
    		
    		Object o2[] = getSubSignalWindow(time, pos, pos+wz);
    		for (int i=0; i< o2.length; i++) t1[i] = (Long) o2[i];
    		
    		detected.add(getSlope(s1, t1));
    	}
    	
    	return detected;
    		
    }
	
	public static Collection<Float> getSlopeCollection(Collection<Float> signal, Collection<Long> time, int wz) {
		
		Float[] s1 = new Float[wz];
		s1 = signal.toArray(s1);
		
		Long[] t1 = new Long[wz];
		t1 = time.toArray(t1);
		
      	return getSlopeCollection(s1, t1, wz);
    		
    }
	
	
	private static Object[] getSubSignalWindow(Object[] signal, int pos1, int pos2) {
		
		Object[] sig = new Object[pos2-pos1+1];
		int i=0;

		for (int p=pos1; p <= pos2; p++) {
			sig[i++] = signal[p];
		}
		
		return sig;		
	}
	
    public static Collection<Float> getAverageCollection(Float[] signal, int wz) {
    	
		ArrayList<Float> detected = new ArrayList<Float>();
		
		if ((signal.length < wz) || (wz < 1)) {
			System.out.println("ERROR: specified window size is invalid!");
			return null;
		}
		
		Float[] s1 = new Float[wz];
		
    	for (int pos=0; pos <= signal.length - wz; pos++) {
    		
    		Object o1[] = getSubSignalWindow(signal, pos, pos+wz-1);
    		for (int i=0; i< o1.length; i++) s1[i] = (Float) o1[i];
    		
    		detected.add(getAverage(s1));
    	}
    	return detected;
    }
    
	public static Collection<Float> getAverageCollection(Collection<Float> signal, int wz) {
		
		Float[] s1 = new Float[wz];
		s1 = signal.toArray(s1);
		
      	return getAverageCollection(s1, wz);
    		
    }
		
    public static Float getAverage(Float[] signal){

    	int sum=0;
    	for (int i=0; i<signal.length; i++) {
    		sum+=signal[i];
    	}
    	
    	return (float)sum/(float)signal.length;

    }
    
    public static Collection<Float> getStdDevCollection(Float[] signal, int wz) {
	  	
		ArrayList<Float> detected = new ArrayList<Float>();
		
		if ((signal.length < wz) || (wz < 1)) {
			System.out.println("ERROR: specified window size is invalid!");
			return null;
		}
		
		Float[] s1 = new Float[wz];
		
    	for (int pos=0; pos <= signal.length - wz; pos++) {
    		
    		Object o1[] = getSubSignalWindow(signal, pos, pos+wz-1);
    		for (int i=0; i< o1.length; i++) s1[i] = (Float) o1[i];
    		detected.add(getStdDev(s1));
    	}
    	
    	return detected;
    }
    
    
	public static Collection<Float> getStdDevCollection(Collection<Float> signal, int wz) {
		
		Float[] s1 = new Float[wz];
		s1 = signal.toArray(s1);
		
      	return getStdDevCollection(s1, wz);
    		
    }
    
    public static Float getStdDev(Float[] signal){
		
    	if (signal.length == 1)
    		return 0f;
    		
    	int n = signal.length;
    	int sum=0;
    	int sum2=0;
    	
    	for (int i=0; i<signal.length; i++) {
    		sum+=signal[i];
    		sum2+=signal[i]*signal[i];
    	}
    	
        float variance = (n*sum2 - sum*sum) / (n*(n-1));
        Double stddev = Math.sqrt(variance);

    	
    	return stddev.floatValue();

    }

    /**
     * Float[] signal: signal to evaluate
     * wz: window size
     */    
    public static Collection<Float> varianceFromStdev(Collection<Float> signal, Collection<Float> mean, Collection<Float> stdDev, int wz) {

    	ArrayList<Float> ret = new ArrayList<Float>();
    	
    	if ((mean.size() != stdDev.size()) || ( signal.size() - wz + 1 != mean.size())) {
    		System.out.println("Invalid set of arguments, sizes don't match");
    		return ret;
    	}
    		
    	ArrayList<Float> sig = (ArrayList<Float>) signal;
    	ArrayList<Float> avg = (ArrayList<Float>) mean;
    	ArrayList<Float> stddev = (ArrayList<Float>) stdDev;
    	
    	for (int indexSignal=wz; indexSignal < sig.size(); indexSignal++) {
    		
    		int index = indexSignal-wz;  		
    		ret.add( (sig.get(indexSignal) - avg.get(index)) / stddev.get(index));
    	}
    	
    	return ret;
    	
    }

    public static Collection<Float> varianceFromStdev(Float[] signal, Collection<Float> mean, Collection<Float> stdDev, int wz) {
    	return varianceFromStdev(new ArrayList(Arrays.asList(signal)), mean, stdDev, wz);
    }
    
    public static Float getPercentile(Float[] signal, int percentile) {
		
    	double[] v = new double[signal.length];
    	
    	for (int i=0; i < signal.length; i++)
    		v[i] = signal[i];
    	
    	Percentile pt = new Percentile();
    	pt.setPercentile(percentile);
    	return (float) pt.evaluate(v);
    	
    }
    
    public static Float getPercentile(Collection<Float> signal, int percentile) {
    	Float[] temp = new Float[signal.size()];
    	return getPercentile(signal.toArray(temp), percentile);
    }
    
    public static Float getPercentileAboveValue(Float[] signal, int percentile, float value) {

    	double[] temp = new double[signal.length];
    	int count=0;

    	for (int i=0; i < signal.length; i++) {
    		if (signal[i] >= value)
	    		temp[count++] = signal[i];
    	}
    	
    	double[] v = new double[count];
    	System.arraycopy(temp, 0, v, 0, count);

    	Percentile pt = new Percentile();
    	pt.setPercentile(percentile);
    	return (float) pt.evaluate(v);
    	
    }
    
    public static Float getPercentileAboveValue(Collection<Float> signal, int percentile, int value) {
    	Float[] temp = new Float[signal.size()];
    	return getPercentileAboveValue(signal.toArray(temp), percentile, value);
    }
    
    public static Float getMax(Float[] signal) {
    	return getMax(new ArrayList(Arrays.asList(signal)));
    }
    
    public static Float getMax(Collection<Float> signal) {
    	return Collections.max(signal);
    }

    public static Float getMin(Float[] signal) {
    	return getMin(new ArrayList(Arrays.asList(signal)));
    }
    
    public static Float getMin(Collection<Float> signal) {
    	return Collections.min(signal);
    }

    public static int getPercentileIndexPosition(Collection<Float> signal, int percentile) {

    	float pctl = getPercentile(signal, percentile);
    	ArrayList<Float> s = new ArrayList(signal);
    	Collections.sort(s);

    	int ret = Math.abs(Collections.binarySearch(s, pctl));
    	int index = (ret<s.size()) ? ret : ret-1;
    	
    	return ((ArrayList) signal).indexOf(s.get(index));
    }
    
    public static int getPercentileIndexPosition(Float[] signal, int percentile) {
    	return getPercentileIndexPosition(new ArrayList(Arrays.asList(signal)), percentile);
    }    
    
    public static Collection<Float> subSignal(Collection<Float> signal, int fromIndex, int numElements) {
    	
    	if (fromIndex < 0 || fromIndex >= signal.size() || fromIndex+numElements > signal.size())
    		return signal;
    	else {
    		return new ArrayList<Float>(((ArrayList) signal).subList(fromIndex, fromIndex+numElements));
    	}
    }
    
    public static Collection<Float> subSignal(Float[] signal, int fromIndex, int numElements) {
    	return subSignal(new ArrayList(Arrays.asList(signal)), fromIndex, numElements);
    }
    
    public static Collection<Float> subSignal(Collection<Float> signal, int fromIndex, int numElements, int intervalLength) {
    	return subSignal(signal, fromIndex/intervalLength, numElements/intervalLength);
    }
    
    public static Collection<Float> subSignal(Float[] signal, int fromIndex, int numElements, int intervalLength) {
    	return subSignal(new ArrayList(Arrays.asList(signal)), fromIndex/intervalLength, numElements/intervalLength);
    }

}

==> dasu_3.02.jar: edu/northwestern/dasu/measurement/SignalManipulation.java <==
/**
 * Dasu Project
 *
 * File:         Mathematics.java
 * RCS:          $Id$
 * Description:  Mathematics class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Oct 10, 2010 at 4:13:22 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.measurement
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.measurement;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;

import org.apache.commons.math.stat.univariate.rank.Percentile;

import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.measurement.NetstatBWStats.NetstatBWData;

/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The Mathematics class ...
 */
public class SignalManipulation {

    /*
     * Does not exist for signals with variance equal to zero
     */
    public static Float getPearsonCorrelation(Float[] scores1,Float[] scores2){
        Double result = 0d;
        Float sum_sq_x = 0f;
        Float sum_sq_y = 0f;
        Float sum_coproduct = 0f;
        Float mean_x = scores1[0];
        Float mean_y = scores2[0];
        for(int i=2;i<scores1.length+1;i+=1){
            Float sweep =Float.valueOf(i-1)/i;
            Float delta_x = scores1[i-1]-mean_x;
            Float delta_y = scores2[i-1]-mean_y;
            sum_sq_x += delta_x * delta_x * sweep;
            sum_sq_y += delta_y * delta_y * sweep;
            sum_coproduct += delta_x * delta_y * sweep;
            mean_x += delta_x / i;
            mean_y += delta_y / i;
        }
       
        Double pop_sd_x = (Double) Math.sqrt(sum_sq_x/scores1.length);
        Double pop_sd_y = (Double) Math.sqrt(sum_sq_y/scores1.length);
        Float cov_x_y = sum_coproduct / scores1.length;
        result = cov_x_y / (pop_sd_x*pop_sd_y);
        return result.floatValue();
    }
    
    public static Float getPearsonCorrelation(Collection<Float> s1, Collection<Float> s2){
    	
    	Float[] scores1 = new Float[s1.size()];
    	scores1 = s1.toArray(scores1);
    	Float[] scores2 = new Float[s2.size()];
    	scores2 = s2.toArray(scores2);

    	return getPearsonCorrelation(scores1, scores2);
    		
    }
    
	public static Collection<Float> getCorrelationCollection(Float[] signal1, Float[] signal2, int wz) {
		
		LinkedList<Float> detected = new LinkedList<Float>();
		
		if (((signal1.length != signal2.length) || (signal1.length) < wz) || (wz < 1)) {
			System.out.println("ERROR: signals are of different length or specified window size is invalid!");
			return null;
		}
		
		Float[] s1 = new Float[wz+1];
		Float[] s2 = new Float[wz+1];
		
    	for (int pos=0; pos < signal1.length - wz; pos++) {
    		//System.out.println("pos: " + pos);
    		
    		Object o1[] = getSubSignalWindow(signal1, pos, pos+wz);
    		for (int i=0; i< o1.length; i++) s1[i] = (Float) o1[i];
    		
    		Object o2[] = getSubSignalWindow(signal2, pos, pos+wz);
    		for (int i=0; i< o2.length; i++) s2[i] = (Float) o2[i];
    		
    		detected.add(getPearsonCorrelation(s1, s2));
    	}
    	
    	return detected;
    		
    }
    
	public static Collection<Float> getCorrelationCollection(Collection<Float> signal1, Collection<Float> signal2, int wz) {
		
		Float[] s1 = new Float[wz];
		Float[] s2 = new Float[wz];
		s1 = signal1.toArray(s1);
		s2 = signal2.toArray(s2);
		
      	return getCorrelationCollection(s1, s2, wz);
    		
    }
	
//	public static Collection<Float> getCumulative(Collection<Float> signal) {
//		
//		LinkedList<Float> ret = new LinkedList<Float>();
//		Iterator<Float> iter = signal.iterator();
//		Float prev = 0f;
//		while (iter.hasNext()) {
//			Float val = iter.next();
//			ret.add(prev+val);
//			prev += val;
//		}
//		
//		return ret;
//		
//	}
	
	public static Collection getCumulative(Collection signal) {
		
		LinkedList ret = new LinkedList();
		Iterator iter = signal.iterator();
		Object obj = iter.next();
		if ( obj instanceof Float) {

			Float prev = (Float) obj;
			ret.add(prev);
			
			while (iter.hasNext()) {
				Float val = (Float) iter.next();
				ret.add(prev+val);
				prev += val;
			}
		}
		
		if ( obj instanceof Long) {

			Long prev = (Long) obj;
			ret.add(prev);
			
			while (iter.hasNext()) {
				Long val = (Long) iter.next();
				ret.add(prev+val);
				prev += val;
			}
		}
		
		return ret;
		
	}
	
	public static Collection<Float> getCumulative(Float[] signal) {
		return getCumulative(new ArrayList(Arrays.asList(signal)));
	}

	public static Collection<Long> getCumulative(Long[] signal) {
		return getCumulative(new ArrayList(Arrays.asList(signal)));
	}
	
    public static Float getSlope(Float[] signal,Long[] time){
    	
    	if (time[time.length-1] == time[0])
    		return null;
    	
    	return (signal[signal.length-1] - signal[0]) / (time[time.length-1] - time[0]);
    }
    
    public static Float getSlope(Collection<Float> signal, Collection<Long> time){

		Float[] s1 = new Float[signal.size()];
		Long[] s2 = new Long[time.size()];
		s1 = signal.toArray(s1);
		s2 = time.toArray(s2);
		
		return getSlope(s1, s2);
    }
	
	public static Collection<Float> getSlopeCollection(Float[] signal, Long[] time, int wz) {
		
		LinkedList<Float> detected = new LinkedList<Float>();
		
		if (((signal.length != time.length) || (signal.length) < wz) || (wz < 2)) {
			System.out.println("ERROR: signals are of different length or specified window size is invalid!");
			return null;
		}
		
		Float[] s1 = new Float[wz+1];
		Long[] t1 = new Long[wz+1];
		
    	for (int pos=0; pos < signal.length - wz; pos++) {
    		
    		Object o1[] = getSubSignalWindow(signal, pos, pos+wz);
    		for (int i=0; i< o1.length; i++) s1[i] = (Float) o1[i];
    		
    		Object o2[] = getSubSignalWindow(time, pos, pos+wz);
    		for (int i=0; i< o2.length; i++) t1[i] = (Long) o2[i];
    		
    		detected.add(getSlope(s1, t1));
    	}
    	
    	return detected;
    		
    }
	
	public static Collection<Float> getSlopeCollection(Collection<Float> signal, Collection<Long> time, int wz) {
		
		Float[] s1 = new Float[wz];
		s1 = signal.toArray(s1);
		
		Long[] t1 = new Long[wz];
		t1 = time.toArray(t1);
		
      	return getSlopeCollection(s1, t1, wz);
    		
    }
	
	
	private static Object[] getSubSignalWindow(Object[] signal, int pos1, int pos2) {
		
		Object[] sig = new Object[pos2-pos1+1];
		int i=0;

		for (int p=pos1; p <= pos2; p++) {
			sig[i++] = signal[p];
		}
		
		return sig;		
	}
	
    public static Collection<Float> getAverageCollection(Float[] signal, int wz) {
    	
		ArrayList<Float> detected = new ArrayList<Float>();
		
		if ((signal.length < wz) || (wz < 1)) {
			System.out.println("ERROR: specified window size is invalid!");
			return null;
		}
		
		Float[] s1 = new Float[wz];
		
    	for (int pos=0; pos <= signal.length - wz; pos++) {
    		
    		Object o1[] = getSubSignalWindow(signal, pos, pos+wz-1);
    		for (int i=0; i< o1.length; i++) s1[i] = (Float) o1[i];
    		
    		detected.add(getAverage(s1));
    	}
    	return detected;
    }
    
	public static Collection<Float> getAverageCollection(Collection<Float> signal, int wz) {
		
		Float[] s1 = new Float[wz];
		s1 = signal.toArray(s1);
		
      	return getAverageCollection(s1, wz);
    		
    }
		
    public static Float getAverage(Float[] signal){

    	int sum=0;
    	for (int i=0; i<signal.length; i++) {
    		sum+=signal[i];
    	}
    	
    	return (float)sum/(float)signal.length;

    }
    
    public static Collection<Float> getStdDevCollection(Float[] signal, int wz) {
	  	
		ArrayList<Float> detected = new ArrayList<Float>();
		
		if ((signal.length < wz) || (wz < 1)) {
			System.out.println("ERROR: specified window size is invalid!");
			return null;
		}
		
		Float[] s1 = new Float[wz];
		
    	for (int pos=0; pos <= signal.length - wz; pos++) {
    		
    		Object o1[] = getSubSignalWindow(signal, pos, pos+wz-1);
    		for (int i=0; i< o1.length; i++) s1[i] = (Float) o1[i];
    		detected.add(getStdDev(s1));
    	}
    	
    	return detected;
    }
    
    
	public static Collection<Float> getStdDevCollection(Collection<Float> signal, int wz) {
		
		Float[] s1 = new Float[wz];
		s1 = signal.toArray(s1);
		
      	return getStdDevCollection(s1, wz);
    		
    }
    
    public static Float getStdDev(Float[] signal){
		
    	if (signal.length == 1)
    		return 0f;
    		
    	int n = signal.length;
    	int sum=0;
    	int sum2=0;
    	
    	for (int i=0; i<signal.length; i++) {
    		sum+=signal[i];
    		sum2+=signal[i]*signal[i];
    	}
    	
        float variance = (n*sum2 - sum*sum) / (n*(n-1));
        Double stddev = Math.sqrt(variance);

    	
    	return stddev.floatValue();

    }

    /**
     * Float[] signal: signal to evaluate
     * wz: window size
     */    
    public static Collection<Float> varianceFromStdev(Collection<Float> signal, Collection<Float> mean, Collection<Float> stdDev, int wz) {

    	ArrayList<Float> ret = new ArrayList<Float>();
    	
    	if ((mean.size() != stdDev.size()) || ( signal.size() - wz + 1 != mean.size())) {
    		System.out.println("Invalid set of arguments, sizes don't match");
    		return ret;
    	}
    		
    	ArrayList<Float> sig = new ArrayList<Float>(signal);
    	ArrayList<Float> avg = new ArrayList<Float>(mean);
    	ArrayList<Float> stddev = new ArrayList<Float>(stdDev);
    	
    	for (int indexSignal=wz; indexSignal < sig.size(); indexSignal++) {
    		
    		int index = indexSignal-wz;  		
    		ret.add( (sig.get(indexSignal) - avg.get(index)) / stddev.get(index));
    	}
    	
    	return ret;
    	
    }

    public static Collection<Float> varianceFromStdev(Float[] signal, Collection<Float> mean, Collection<Float> stdDev, int wz) {
    	return varianceFromStdev(new ArrayList(Arrays.asList(signal)), mean, stdDev, wz);
    }
    
    public static Float getPercentile(Float[] signal, int percentile) {
		
    	double[] v = new double[signal.length];
    	
    	for (int i=0; i < signal.length; i++)
    		v[i] = signal[i];
    	
    	Percentile pt = new Percentile();
    	pt.setPercentile(percentile);
    	return (float) pt.evaluate(v);
    	
    }
    
    public static Float getPercentile(Collection<Float> signal, int percentile) {
    	Float[] temp = new Float[signal.size()];
    	return getPercentile(signal.toArray(temp), percentile);
    }
    
    public static Float getPercentileAboveValue(Float[] signal, int percentile, float value) {

    	double[] temp = new double[signal.length];
    	int count=0;

    	for (int i=0; i < signal.length; i++) {
    		if (signal[i] >= value)
	    		temp[count++] = signal[i];
    	}
    	
    	double[] v = new double[count];
    	System.arraycopy(temp, 0, v, 0, count);

    	Percentile pt = new Percentile();
    	pt.setPercentile(percentile);
    	return (float) pt.evaluate(v);
    	
    }
    
    public static Float getPercentileAboveValue(Collection<Float> signal, int percentile, int value) {
    	Float[] temp = new Float[signal.size()];
    	return getPercentileAboveValue(signal.toArray(temp), percentile, value);
    }
    
    public static Float getMax(Float[] signal) {
    	return getMax(new ArrayList(Arrays.asList(signal)));
    }
    
    public static Float getMax(Collection<Float> signal) {
    	return Collections.max(signal);
    }

    public static Float getMin(Float[] signal) {
    	return getMin(new ArrayList(Arrays.asList(signal)));
    }
    
    public static Float getMin(Collection<Float> signal) {
    	return Collections.min(signal);
    }

    public static int getPercentileIndexPosition(Collection<Float> signal, int percentile) {

    	float pctl = getPercentile(signal, percentile);
    	ArrayList<Float> s = new ArrayList(signal);
    	Collections.sort(s);

    	int ret = Math.abs(Collections.binarySearch(s, pctl));
    	int index = (ret<s.size()) ? ret : ret-1;
    	
    	ArrayList<Float> sig = new ArrayList<Float>(signal);
    	return sig.indexOf(s.get(index));
    }
    
    public static int getPercentileIndexPosition(Float[] signal, int percentile) {
    	return getPercentileIndexPosition(new ArrayList(Arrays.asList(signal)), percentile);
    }    
    
    public static Collection<Float> subSignal(Collection<Float> signal, int fromIndex, int numElements) {
    	
    	if (fromIndex < 0 || fromIndex >= signal.size() || fromIndex+numElements > signal.size())
    		return signal;
    	else {
    		ArrayList<Float> sig = new ArrayList<Float>(signal);
    		return new ArrayList<Float>(sig.subList(fromIndex, fromIndex+numElements));
    	}
    }
    
    public static Collection<Float> subSignal(Float[] signal, int fromIndex, int numElements) {
    	return subSignal(new ArrayList(Arrays.asList(signal)), fromIndex, numElements);
    }
    
    public static Collection<Float> subSignal(Collection<Float> signal, int fromIndex, int numElements, int intervalLength) {
    	return subSignal(signal, fromIndex/intervalLength, numElements/intervalLength);
    }
    
    public static Collection<Float> subSignal(Float[] signal, int fromIndex, int numElements, int intervalLength) {
    	return subSignal(new ArrayList(Arrays.asList(signal)), fromIndex/intervalLength, numElements/intervalLength);
    }

}
